@@ -23,40 +23,61 @@
 #ifndef _ASTERISK_COMPILER_H
 #define _ASTERISK_COMPILER_H
 
-#if HAVE_ATTRIBUTE_always_inline
+#ifdef HAVE_ATTRIBUTE_always_inline
 #define force_inline __attribute__((always_inline)) inline
 #else
 #define force_inline inline
 #endif
 
-#if HAVE_ATTRIBUTE_pure
+#ifdef HAVE_ATTRIBUTE_pure
 #define attribute_pure __attribute__((pure))
 #else
 #define attribute_pure
 #endif
 
-#if HAVE_ATTRIBUTE_const
+#ifdef HAVE_ATTRIBUTE_const
 #define attribute_const __attribute__((const))
 #else
 #define attribute_const
 #endif
 
-#if HAVE_ATTRIBUTE_unused
+#ifdef HAVE_ATTRIBUTE_unused
 #define attribute_unused __attribute__((unused))
 #else
 #define attribute_unused
 #endif
 
-#if HAVE_ATTRIBUTE_malloc
+#ifdef HAVE_ATTRIBUTE_malloc
 #define attribute_malloc __attribute__((malloc))
 #else
 #define attribute_malloc
 #endif
 
-#if HAVE_ATTRIBUTE_deprecated
-#define attribute_deprecated __attribute__((deprecated))
+#ifdef HAVE_ATTRIBUTE_sentinel
+#define attribute_sentinel __attribute__((sentinel))
 #else
-#define attribute_deprecated
+#define attribute_sentinel
+#endif
+
+#ifdef HAVE_ATTRIBUTE_warn_unused_result
+#define attribute_warn_unused_result __attribute__((warn_unused_result))
+#else
+#define attribute_warn_unused_result
+#endif
+
+/* Some older version of GNU gcc (3.3.5 on OpenBSD 4.3 for example) dont like 'NULL' as sentinel */
+#define SENTINEL ((char *)NULL)
+
+#ifdef HAVE_ATTRIBUTE_weak
+#define attribute_weak __attribute__((weak))
+#else
+#define attribute_weak
+#endif
+
+#ifdef HAVE_ATTRIBUTE_weak_import
+#define attribute_weak_import __attribute__((weak_import))
+#else
+#define attribute_weak_import
 #endif
 
 #endif /* _ASTERISK_COMPILER_H */
